#!/usr/bin/env python3
"""
Handles risk management checks and cleanup routines for orphaned orders.
"""

import logging
import asyncio
from datetime import datetime, timezone
from typing import Optional, Dict, Any, List

from src.config.config import Config # For STOP_LOSS_PERCENTAGE
from src.utils.token_display_formatter import get_formatter

logger = logging.getLogger(__name__)

class RiskCleanupManager:
    def __init__(self, trading_engine, notification_manager, market_monitor_cache, shared_state):
        self.trading_engine = trading_engine
        self.notification_manager = notification_manager
        self.market_monitor_cache = market_monitor_cache # To access cached orders/positions
        self.shared_state = shared_state # Store shared_state
        # Initialize external_stop_losses within shared_state if it's not already there
        if 'external_stop_losses' not in self.shared_state:
            self.shared_state['external_stop_losses'] = {}
        # For direct access if needed, though primary access should be via shared_state
        self.external_stop_losses: Dict[str, Dict[str, Any]] = self.shared_state['external_stop_losses']

    # Methods like _check_automatic_risk_management, _cleanup_orphaned_stop_losses,
    # _check_external_stop_loss_orders, _cleanup_external_stop_loss_tracking,
    # _check_pending_triggers, and the new _cleanup_orphaned_pending_sl_activations will go here
    pass 

    async def _check_pending_triggers(self):
        """Check and process pending conditional triggers (e.g., SL/TP)."""
        stats = self.trading_engine.get_stats()
        if not stats:
            logger.warning("⚠️ TradingStats not available in _check_pending_triggers.")
            return

        try:
            pending_sl_triggers = stats.get_orders_by_status(status='pending_trigger', order_type_filter='stop_limit_trigger')
            
            if not pending_sl_triggers:
                return

            logger.info(f"Found {len(pending_sl_triggers)} pending SL triggers to check.")

            for trigger_order in pending_sl_triggers:
                symbol = trigger_order['symbol']
                trigger_price = trigger_order['price']
                trigger_side = trigger_order['side']
                order_db_id = trigger_order['id']
                parent_ref_id = trigger_order.get('parent_bot_order_ref_id')

                if not symbol or trigger_price is None:
                    logger.warning(f"Invalid trigger order data for DB ID {order_db_id}, skipping: {trigger_order}")
                    continue

                market_data = await self.trading_engine.get_market_data(symbol)
                if not market_data or not market_data.get('ticker'):
                    logger.warning(f"Could not fetch market data for {symbol} to check SL trigger {order_db_id}.")
                    continue
                
                current_price = float(market_data['ticker'].get('last', 0))
                if current_price <= 0:
                    logger.warning(f"Invalid current price ({current_price}) for {symbol} checking SL trigger {order_db_id}.")
                    continue

                trigger_hit = False
                if trigger_side.lower() == 'sell' and current_price <= trigger_price:
                    trigger_hit = True
                    logger.info(f"🔴 SL TRIGGER HIT (Sell): Order DB ID {order_db_id}, Symbol {symbol}, Trigger@ ${trigger_price:.4f}, Market@ ${current_price:.4f}")
                elif trigger_side.lower() == 'buy' and current_price >= trigger_price:
                    trigger_hit = True
                    logger.info(f"🟢 SL TRIGGER HIT (Buy): Order DB ID {order_db_id}, Symbol {symbol}, Trigger@ ${trigger_price:.4f}, Market@ ${current_price:.4f}")
                
                if trigger_hit:
                    # Verify active position before executing SL
                    active_position_exists = False
                    cached_positions = self.market_monitor_cache.cached_positions or []
                    for pos in cached_positions:
                        if pos.get('symbol') == symbol and abs(float(pos.get('contracts') or 0)) > 1e-9: # Check for non-zero contracts
                            active_position_exists = True
                            break
                    
                    if not active_position_exists:
                        logger.warning(f"🚫 SL TRIGGER IGNORED for {symbol} (DB ID: {order_db_id}): No active position found. Cancelling trigger.")
                        stats.update_order_status(order_db_id=order_db_id, new_status='cancelled_no_position')
                        if self.notification_manager:
                            await self.notification_manager.send_generic_notification(
                                f"⚠️ Stop-Loss Trigger Cancelled (No Position)\\n"
                                f"Symbol: {symbol}\\n"
                                f"Side: {trigger_side.upper()}\\n"
                                f"Trigger Price: ${trigger_price:.4f}\\n"
                                f"Reason: The stop-loss trigger price was hit, but no active position was found for this symbol.\\n"
                                f"The pending SL (DB ID: {order_db_id}) has been cancelled to prevent incorrect order placement."
                            )
                        continue # Skip to the next trigger

                    logger.info(f"Attempting to execute actual stop order for triggered DB ID: {order_db_id} (Parent Bot Ref: {trigger_order.get('parent_bot_order_ref_id')})")
                    execution_result = await self.trading_engine.execute_triggered_stop_order(original_trigger_order_db_id=order_db_id)
                    notification_message_detail = ""

                    if execution_result.get("success"):
                        new_trigger_status = 'triggered_order_placed'
                        placed_sl_details = execution_result.get("placed_sl_order_details", {})
                        logger.info(f"Successfully placed actual SL order from trigger {order_db_id}. New SL Order DB ID: {placed_sl_details.get('order_db_id')}, Exchange ID: {placed_sl_details.get('exchange_order_id')}")
                        notification_message_detail = f"Actual SL order placed (New DB ID: {placed_sl_details.get('order_db_id', 'N/A')})."
                    else:
                        new_trigger_status = 'trigger_execution_failed'
                        error_msg = execution_result.get("error", "Unknown error during SL execution.")
                        logger.error(f"Failed to execute actual SL order from trigger {order_db_id}: {error_msg}")
                        notification_message_detail = f"Failed to place actual SL order: {error_msg}"

                    stats.update_order_status(order_db_id=order_db_id, new_status=new_trigger_status)
                    
                    if self.notification_manager:
                        await self.notification_manager.send_generic_notification(
                            f"🔔 Stop-Loss Update!\\nSymbol: {symbol}\\nSide: {trigger_side.upper()}\\nTrigger Price: ${trigger_price:.4f}\\nMarket Price: ${current_price:.4f}\\n(Original Trigger DB ID: {order_db_id}, Parent: {parent_ref_id or 'N/A'})\\nStatus: {new_trigger_status.replace('_', ' ').title()}\\nDetails: {notification_message_detail}"
                        )
        except Exception as e:
            logger.error(f"❌ Error checking pending SL triggers: {e}", exc_info=True)

    async def _check_automatic_risk_management(self):
        """Check for automatic stop loss triggers based on Config.STOP_LOSS_PERCENTAGE as safety net."""
        try:
            if not getattr(Config, 'RISK_MANAGEMENT_ENABLED', True) or Config.STOP_LOSS_PERCENTAGE <= 0:
                logger.info(f"Risk management disabled or STOP_LOSS_PERCENTAGE <= 0 (value: {Config.STOP_LOSS_PERCENTAGE})")
                return

            # Use DB positions for risk checks to ensure validated, up-to-date data
            stats = self.trading_engine.get_stats()
            positions = stats.get_open_positions() if stats else []
            if not positions:
                logger.debug("No open positions found in DB for risk management check.")
                await self._cleanup_orphaned_stop_losses() # Call within class
                return

            for position in positions:
                try:
                    symbol = position.get('symbol', '')
                    
                    # Safely convert position values, handling None values
                    contracts = float(position.get('current_position_size') or 0)
                    entry_price = float(position.get('entry_price') or 0)
                    roe_percentage = float(position.get('roe_percentage') or 0)
                    unrealized_pnl = float(position.get('unrealized_pnl') or 0)

                    if contracts == 0 or entry_price <= 0:
                        logger.info(f"Skipping position {symbol}: contracts={contracts}, entry_price={entry_price}")
                        continue

                    logger.debug(f"[RiskMgmt] {symbol}: ROE={roe_percentage:+.2f}%, Threshold=-{Config.STOP_LOSS_PERCENTAGE}% (Trigger: {roe_percentage <= -Config.STOP_LOSS_PERCENTAGE})")

                    if roe_percentage <= -Config.STOP_LOSS_PERCENTAGE:
                        token = symbol.split('/')[0] if '/' in symbol else symbol.split(':')[0]
                        position_side = "LONG" if contracts > 0 else "SHORT"
                        stats = self.trading_engine.get_stats()
                        lifecycle_id_str = "N/A"
                        if stats:
                            active_trade_lc = stats.get_trade_by_symbol_and_status(symbol, 'position_opened')
                            if active_trade_lc:
                                lifecycle_id_str = active_trade_lc.get('trade_lifecycle_id', "N/A")[:8] + "..."

                        logger.warning(f"🚨 AUTOMATIC STOP LOSS TRIGGERED: {token} {position_side} position (Lifecycle: {lifecycle_id_str}) has {roe_percentage:+.2f}% ROE loss (threshold: -{Config.STOP_LOSS_PERCENTAGE}%)")
                        
                        if self.notification_manager:
                            await self.notification_manager.send_generic_notification(
                                f"""🚨 AUTOMATIC STOP LOSS TRIGGERED!

Token: {token}
Lifecycle ID: {lifecycle_id_str}
Position: {position_side} {abs(contracts):.6f}
Entry Price: ${entry_price:.4f}
Unrealized P&L: ${unrealized_pnl:+.2f} ({roe_percentage:+.2f}% ROE)
Safety Threshold: -{Config.STOP_LOSS_PERCENTAGE}% ROE
Action: Executing emergency exit order..."""
                            )

                        # Execute emergency exit order
                        exit_result = await self.trading_engine.execute_exit_order(token)
                        if not exit_result.get('success'):
                            error_msg = exit_result.get('error', 'Unknown error')
                            logger.error(f"❌  Failed to execute emergency exit order for {token} (Lifecycle: {lifecycle_id_str}): {error_msg}")
                            if self.notification_manager:
                                await self.notification_manager.send_generic_notification(
                                    f"⚠️ <b>Emergency Exit Failed</b>\n\n"
                                    f"Token: {token}\n"
                                    f"Position: {position_side.upper()}\n"
                                    f"ROE: {roe_percentage:.2f}%\n"
                                    f"Error: {error_msg}\n\n"
                                    f"Please check the position and close it manually if needed."
                                )
                            continue

                        # Cancel any pending stop losses for this symbol
                        if self.trading_engine.stats and hasattr(self.trading_engine.stats.order_manager, 'cancel_pending_stop_losses_by_symbol'):
                            try:
                                cancelled_sl_count = self.trading_engine.stats.order_manager.cancel_pending_stop_losses_by_symbol(
                                    symbol=symbol,
                                    new_status='cancelled_automatic_exit'
                                )
                                if cancelled_sl_count > 0:
                                    logger.info(f"Cancelled {cancelled_sl_count} pending stop losses for {symbol} after automatic exit")
                            except Exception as sl_error:
                                logger.error(f"Error cancelling pending stop losses for {symbol}: {sl_error}")
                except Exception as pos_error:
                    logger.error(f"Error processing position for automatic stop loss: {pos_error}")
                    continue
        except Exception as e:
            logger.error(f"❌ Error in automatic risk management check: {e}", exc_info=True)

    async def _cleanup_orphaned_stop_losses(self):
        """Clean up pending stop losses that no longer have corresponding positions OR whose parent orders have been cancelled/failed."""
        try:
            stats = self.trading_engine.get_stats()
            if not stats:
                return

            pending_stop_losses = stats.get_orders_by_status('pending_trigger', 'stop_limit_trigger')
            if not pending_stop_losses:
                return

            logger.info(f"Checking {len(pending_stop_losses)} pending stop losses for orphaned orders")
            current_positions = self.market_monitor_cache.cached_positions or [] 
            position_symbols = set()
            if current_positions:
                for pos in current_positions:
                    symbol = pos.get('symbol')
                    contracts = float(pos.get('contracts') or 0)
                    if symbol and contracts != 0:
                        position_symbols.add(symbol)

            orphaned_count = 0
            for sl_order in pending_stop_losses:
                symbol = sl_order.get('symbol')
                order_db_id = sl_order.get('id')
                parent_bot_ref_id = sl_order.get('parent_bot_order_ref_id')
                should_cancel = False
                cancel_reason = ""
                
                if parent_bot_ref_id:
                    parent_order = stats.get_order_by_bot_ref_id(parent_bot_ref_id)
                    if parent_order:
                        parent_status = parent_order.get('status', '').lower()
                        if parent_order.get('exchange_order_id'):
                            entry_oid = parent_order['exchange_order_id']
                            lc_pending = stats.get_lifecycle_by_entry_order_id(entry_oid, status='pending')
                            lc_cancelled = stats.get_lifecycle_by_entry_order_id(entry_oid, status='cancelled')
                            lc_opened = stats.get_lifecycle_by_entry_order_id(entry_oid, status='position_opened')

                            if parent_status == 'cancelled_externally':
                                if lc_cancelled:
                                    should_cancel = True
                                    cancel_reason = f"parent order ({entry_oid[:6]}...) {parent_status} and lifecycle explicitly cancelled"
                                elif not lc_pending and not lc_opened:
                                    should_cancel = True
                                    cancel_reason = f"parent order ({entry_oid[:6]}...) {parent_status} and no active/pending/cancelled lifecycle found"
                                else:
                                    current_lc_status = "N/A"
                                    if lc_pending: current_lc_status = lc_pending.get('status')
                                    elif lc_opened: current_lc_status = lc_opened.get('status')
                                    logger.info(f"SL {order_db_id} for parent {parent_bot_ref_id} (status {parent_status}) - lifecycle is '{current_lc_status}'. SL not cancelled by this rule.")
                                    should_cancel = False
                            elif parent_status in ['failed_submission', 'failed_submission_no_data', 'cancelled_manually', 'disappeared_from_exchange']:
                                if not lc_opened: 
                                    should_cancel = True
                                    cancel_reason = f"parent order ({entry_oid[:6]}...) {parent_status} and no 'position_opened' lifecycle"
                                else:
                                    logger.info(f"SL {order_db_id} for parent {parent_bot_ref_id} (status {parent_status}) - lifecycle is 'position_opened'. SL not cancelled by this rule.")
                                    should_cancel = False
                            elif parent_status == 'filled':
                                if symbol not in position_symbols: 
                                    should_cancel = True
                                    cancel_reason = "parent filled but actual position no longer exists"
                        else: 
                            if parent_status in ['open', 'pending_submission', 'submitted']:
                                logger.info(f"SL Cleanup: Parent order {parent_order.get('id')} (status '{parent_status}') is missing exchange_id in DB record. SL {sl_order.get('id')} will NOT be cancelled by this rule, assuming parent is still active or pending.")
                                should_cancel = False
                                if parent_status == 'open':
                                    logger.warning(f"SL Cleanup: DATA ANOMALY - Parent order {parent_order.get('id')} status is 'open' but fetched DB record shows no exchange_id. Investigate DB state for order {parent_order.get('id')}.)")
                            else:
                                should_cancel = True
                                cancel_reason = f"parent order {parent_status} (no exch_id) and status indicates it's not live/pending."
                    else: 
                        should_cancel = True
                        cancel_reason = "parent order not found in database"
                else:
                    if symbol not in position_symbols:
                        should_cancel = True
                        cancel_reason = "no position exists and no parent reference"
                
                if should_cancel:
                    success = stats.update_order_status(
                        order_db_id=order_db_id,
                        new_status='cancelled_orphaned'
                    )
                    if success:
                        orphaned_count += 1
                        token = symbol.split('/')[0] if '/' in symbol else symbol.split(':')[0]
                        logger.info(f"🧹 Cancelled orphaned stop loss for {token} (Order DB ID: {order_db_id}) - Reason: {cancel_reason}")

            if orphaned_count > 0:
                logger.info(f"🧹 Cleanup completed: Cancelled {orphaned_count} orphaned stop loss order(s)")
                if self.notification_manager:
                    await self.notification_manager.send_generic_notification(
                        f"""🧹 <b>Cleanup Completed</b>\\n\\n
Cancelled {orphaned_count} orphaned stop loss order(s)\\n
Reason: Parent orders invalid or positions closed externally\\n
Time: {datetime.now(timezone.utc).strftime('%H:%M:%S')}\\n\\n
💡 This ensures stop losses sync with actual orders/positions."""
                    )
        except Exception as e:
            logger.error(f"❌ Error cleaning up orphaned stop losses: {e}", exc_info=True)

    async def _check_external_stop_loss_orders(self):
        """Check for externally placed stop loss orders and track them."""
        try:
            open_orders = self.market_monitor_cache.cached_orders or []
            if not open_orders:
                return
            positions = self.market_monitor_cache.cached_positions or []
            if not positions:
                return
            
            position_map = {}
            for position in positions:
                symbol = position.get('symbol')
                contracts = float(position.get('contracts') or 0)
                if symbol and contracts != 0:
                    token_map_key = symbol.split('/')[0] if '/' in symbol else symbol.split(':')[0]
                    position_map[token_map_key] = {
                        'symbol': symbol,
                        'contracts': contracts,
                        'side': 'long' if contracts > 0 else 'short',
                        'entry_price': float(position.get('entryPx') or 0)
                    }
            
            newly_detected = 0
            formatter = get_formatter()
            for order in open_orders:
                try:
                    exchange_order_id = order.get('id')
                    symbol = order.get('symbol')
                    order_side = order.get('side', '').lower()
                    order_type = order.get('type', '').lower()
                    order_price = float(order.get('price', 0))
                    stop_price = float(order.get('info', {}).get('triggerPrice', 0)) # Hyperliquid specific
                    is_reduce_only = order.get('reduceOnly', False) or order.get('info',{}).get('reduceOnly',False)
                    order_amount = float(order.get('amount',0))

                    if not (symbol and exchange_order_id and order_side and order_type):
                        continue
                    
                    token = symbol.split('/')[0] if '/' in symbol else symbol.split(':')[0]

                    if exchange_order_id in self.external_stop_losses: # Already tracking
                        continue

                    if token in position_map:
                        pos_details = position_map[token]
                        is_potential_sl = False
                        if pos_details['side'] == 'long' and order_side == 'sell':
                            if stop_price > 0 and stop_price < pos_details['entry_price']:
                                is_potential_sl = True
                            elif order_type in ['stop', 'stopmarket', 'stop-market'] and order_price > 0 and order_price < pos_details['entry_price']:
                                is_potential_sl = True # Using order_price as trigger if stop_price is missing for some reason
                        elif pos_details['side'] == 'short' and order_side == 'buy':
                            if stop_price > 0 and stop_price > pos_details['entry_price']:
                                is_potential_sl = True
                            elif order_type in ['stop', 'stopmarket', 'stop-market'] and order_price > 0 and order_price > pos_details['entry_price']:
                                is_potential_sl = True
                        
                        if is_potential_sl and is_reduce_only and abs(order_amount - abs(pos_details['contracts'])) < 0.01 * abs(pos_details['contracts']):
                            self.external_stop_losses[exchange_order_id] = {
                                'symbol': symbol, 'side': order_side, 'amount': order_amount,
                                'trigger_price': stop_price or order_price, 'type': order_type,
                                'position_entry_price': pos_details['entry_price'],
                                'position_side': pos_details['side'],
                                'detected_at': datetime.now(timezone.utc).isoformat()
                            }
                            newly_detected += 1
                            logger.info(f"🛡️ Detected external Stop Loss for {token} ({pos_details['side']}): OID {exchange_order_id}, Trigger@ {formatter.format_price(stop_price or order_price, symbol)}, Amount {formatter.format_amount(order_amount, token)}. Now tracking.")
                except Exception as order_err:
                    logger.error(f"Error processing order {order.get('id')} for external SL detection: {order_err}")
                    continue
            
            if newly_detected > 0:
                logger.info(f"🛡️ Detected and now tracking {newly_detected} new external stop loss order(s).")
                # Persist self.external_stop_losses if desired, or rely on periodic checks

        except Exception as e:
            logger.error(f"❌ Error checking external stop loss orders: {e}", exc_info=True)

    async def _cleanup_external_stop_loss_tracking(self):
        """Clean up tracked external stop losses if position is gone or SL order is gone."""
        try:
            if not self.external_stop_losses:
                return

            current_positions = self.market_monitor_cache.cached_positions or []
            position_symbols = {pos.get('symbol') for pos in current_positions if pos.get('symbol') and abs(float(pos.get('contracts') or 0)) > 1e-9}
            
            current_open_orders_on_exchange = self.market_monitor_cache.cached_orders or []
            open_exchange_order_ids = {order.get('id') for order in current_open_orders_on_exchange if order.get('id')}

            removed_count = 0
            for exch_oid, sl_info in list(self.external_stop_losses.items()): # Iterate over a copy
                symbol = sl_info['symbol']
                should_remove = False
                reason = ""

                if symbol not in position_symbols:
                    should_remove = True
                    reason = f"Position for {symbol} no longer exists."
                elif exch_oid not in open_exchange_order_ids:
                    should_remove = True
                    reason = f"SL order {exch_oid} for {symbol} no longer on exchange."
                
                if should_remove:
                    del self.external_stop_losses[exch_oid]
                    removed_count += 1
                    logger.info(f"🧹 External SL Cleanup: Removed tracking for {symbol} SL OID {exch_oid}. Reason: {reason}")
            
            if removed_count > 0:
                logger.info(f"🧹 External SL Cleanup: Removed {removed_count} tracked external stop losses.")

        except Exception as e:
            logger.error(f"❌ Error cleaning up external stop loss tracking: {e}", exc_info=True)

    async def _cleanup_orphaned_pending_sl_activations(self):
        """Clean up 'pending_sl_activation' orders if parent entry order is gone and no position exists for the symbol."""
        try:
            stats = self.trading_engine.get_stats()
            if not stats:
                logger.warning("⚠️ TradingStats not available in _cleanup_orphaned_pending_sl_activations.")
                return

            pending_sl_activation_orders = stats.get_orders_by_status(
                status='pending_activation',
                order_type_filter='pending_sl_activation'
            )

            if not pending_sl_activation_orders:
                return

            logger.info(f"Found {len(pending_sl_activation_orders)} 'pending_sl_activation' orders to check for cleanup.")
            
            # Get current open orders on the exchange from the cache
            open_exchange_orders_map = {
                order.get('id'): order for order in (self.market_monitor_cache.cached_orders or []) if order.get('id')
            }
            # Get current open positions on the exchange from the cache
            active_position_symbols = {
                pos.get('symbol') for pos in (self.market_monitor_cache.cached_positions or [])
                if pos.get('symbol') and abs(float(pos.get('contracts') or 0)) > 1e-9
            }

            cleaned_count = 0
            for sl_order in pending_sl_activation_orders:
                pending_sl_db_id = sl_order.get('id')
                symbol = sl_order.get('symbol')
                parent_bot_ref_id = sl_order.get('parent_bot_order_ref_id')

                if not symbol or not parent_bot_ref_id:
                    logger.warning(f"Skipping pending SL (DB ID: {pending_sl_db_id}) due to missing symbol or parent_bot_ref_id: {sl_order}")
                    continue
                
                # Condition 1: No active position for the symbol
                if symbol in active_position_symbols:
                    logger.info(f"Pending SL (DB ID: {pending_sl_db_id}, Symbol: {symbol}) skipped: Active position exists.")
                    continue

                # Condition 2: Parent entry order is no longer active on the exchange
                parent_order_active_on_exchange = False
                parent_order_in_db = stats.get_order_by_bot_ref_id(parent_bot_ref_id)

                if parent_order_in_db:
                    parent_exchange_order_id = parent_order_in_db.get('exchange_order_id')
                    if parent_exchange_order_id:
                        if parent_exchange_order_id in open_exchange_orders_map:
                            parent_order_active_on_exchange = True
                            logger.info(f"Pending SL (DB ID: {pending_sl_db_id}, Symbol: {symbol}) - Parent order {parent_bot_ref_id} (Exch ID: {parent_exchange_order_id}) is still active on exchange.")
                        else:
                            logger.info(f"Pending SL (DB ID: {pending_sl_db_id}, Symbol: {symbol}) - Parent order {parent_bot_ref_id} (Exch ID: {parent_exchange_order_id}) NOT active on exchange.")
                    else:
                        # Parent order in DB but has no exchange ID (e.g., failed submission, or never placed)
                        logger.info(f"Pending SL (DB ID: {pending_sl_db_id}, Symbol: {symbol}) - Parent order {parent_bot_ref_id} found in DB but has no exchange_order_id. Assuming not active on exchange.")
                else:
                    # Parent order not even found in DB
                    logger.info(f"Pending SL (DB ID: {pending_sl_db_id}, Symbol: {symbol}) - Parent order {parent_bot_ref_id} NOT found in DB. Assuming not active.")

                if not parent_order_active_on_exchange:
                    # Both conditions met: No active position AND parent order is not active
                    logger.info(f"🧹 Cleaning up orphaned 'pending_sl_activation' (DB ID: {pending_sl_db_id}) for symbol {symbol}. Parent order {parent_bot_ref_id} gone and no active position.")
                    success = stats.update_order_status(
                        order_db_id=pending_sl_db_id,
                        new_status='cancelled_orphaned_entry_gone'
                    )
                    if success:
                        cleaned_count += 1
                        logger.info(f"✅ Successfully cancelled orphaned 'pending_sl_activation' (DB ID: {pending_sl_db_id}).")
                        if self.notification_manager:
                            await self.notification_manager.send_generic_notification(
                                f"🧹 <b>Pending SL Activation Cleaned Up</b>\\n\\n"
                                f"Symbol: {symbol}\\n"
                                f"Reason: Original entry order is no longer active and no position exists for this symbol.\\n"
                                f"Pending SL Bot Ref ID: {sl_order.get('bot_order_ref_id', 'N/A')}\\n"
                                f"Parent Entry Bot Ref ID: {parent_bot_ref_id}\\n"
                                f"Time: {datetime.now(timezone.utc).strftime('%Y-%m-%d %H:%M:%S UTC')}"
                            )
                    else:
                        logger.error(f"❌ Failed to cancel orphaned 'pending_sl_activation' (DB ID: {pending_sl_db_id}).")
            
            if cleaned_count > 0:
                logger.info(f"🧹 Pending SL Activation Cleanup: Cancelled {cleaned_count} orphaned 'pending_sl_activation' orders.")

        except Exception as e:
            logger.error(f"❌ Error in _cleanup_orphaned_pending_sl_activations: {e}", exc_info=True)

    async def run_cleanup_tasks(self):
        """Run all risk and cleanup tasks."""
        stats = self.trading_engine.get_stats()
        if not stats:
            logger.warning("⚠️ TradingStats not available, skipping cleanup tasks.")
            return

        await self._check_automatic_risk_management()
        await self._cleanup_orphaned_stop_losses()
        await self._handle_pending_sl_activations(stats)
        await self._cleanup_orphaned_pending_trades(stats)

    async def _handle_pending_sl_activations(self, stats):
        """Handle pending stop losses - place orders for positions that need them."""
        try:
            # Get positions with pending SLs using existing manager
            pending_sl_trades = stats.get_pending_stop_loss_activations()
            
            for trade in pending_sl_trades:
                symbol = trade['symbol']
                stop_price = trade['stop_loss_price']
                position_side = trade['position_side']
                lifecycle_id = trade['trade_lifecycle_id']
                
                try:
                    # Check if position still exists on exchange
                    exchange_positions = self.trading_engine.get_positions()
                    if exchange_positions is None:
                        logger.warning("⚠️ Failed to fetch exchange positions - skipping stop loss check")
                        continue
                    position_exists = any(
                        pos['symbol'] == symbol and abs(float(pos.get('contracts') or 0)) > 1e-9 
                        for pos in exchange_positions
                    )
                    
                    if position_exists:
                        # Place stop loss order
                        sl_side = 'sell' if position_side == 'long' else 'buy'
                        
                        token = symbol.split('/')[0] if '/' in symbol else symbol.split(':')[0]
                        result = await self.trading_engine.execute_stop_loss_order(
                            token=token,
                            stop_price=stop_price
                        )
                        
                        if result and result.get('success'):
                            exchange_order_id = result.get('order_placed_details', {}).get('exchange_order_id')
                            if exchange_order_id:
                                # The execute_stop_loss_order already links the SL to the trade
                                logger.info(f"✅ Placed pending SL: {symbol} @ {stop_price}")
                            else:
                                logger.warning(f"⚠️ SL placed for {symbol} but no exchange_order_id returned")
                    else:
                        # Position doesn't exist, clear pending SL
                        logger.info(f"🗑️ Clearing pending SL for non-existent position: {symbol}")
                        # This will be handled by position closed detection
                        
                except Exception as e:
                    logger.error(f"❌ Error handling pending SL for {symbol}: {e}")
                    
        except Exception as e:
            logger.error(f"❌ Error handling pending stop losses: {e}")

    async def _cleanup_orphaned_pending_trades(self, stats):
        """Handle trades stuck in 'pending' status due to cancelled orders."""
        try:
            # Get all pending trades
            pending_trades = stats.get_trades_by_status('pending')
            
            if not pending_trades:
                return
                
            logger.debug(f"🔍 Checking {len(pending_trades)} pending trades for orphaned status")
            
            # Get current exchange orders for comparison
            exchange_orders = self.trading_engine.get_orders() or []
            exchange_order_ids = {order.get('id') for order in exchange_orders if order.get('id')}
            
            for trade in pending_trades:
                lifecycle_id = trade['trade_lifecycle_id']
                entry_order_id = trade.get('entry_order_id')
                symbol = trade['symbol']

                # If the order is still open on the exchange, it's not orphaned.
                if entry_order_id and entry_order_id in exchange_order_ids:
                    logger.info(f"Trade {lifecycle_id} for {symbol} is pending but its order {entry_order_id} is still open. Skipping.")
                    continue

                # If no order is linked, or the order is not on the exchange,
                # we assume it was cancelled or failed.
                logger.warning(f"Orphaned pending trade detected for {symbol} (Lifecycle: {lifecycle_id}). Cancelling.")

                # Mark the trade as cancelled (this is a sync function)
                cancelled = stats.update_trade_cancelled(lifecycle_id, "Orphaned pending trade")
                
                if cancelled:
                    # Migrate the cancelled trade to aggregated stats
                    stats.migrate_trade_to_aggregated_stats(lifecycle_id)
                    
                    # Send a notification
                    await self._send_trade_cancelled_notification(
                        symbol, "Orphaned, presumed cancelled before fill.", trade
                    )
                else:
                    logger.error(f"Failed to cancel orphaned trade lifecycle {lifecycle_id}")

        except Exception as e:
            logger.error(f"❌ Error handling orphaned pending trades: {e}", exc_info=True)

    async def _send_trade_cancelled_notification(self, symbol: str, cancel_reason: str, trade: Dict[str, Any]):
        """Send notification for a cancelled trade."""
        try:
            if not self.notification_manager:
                return
            
            token = symbol.split('/')[0] if '/' in symbol else symbol.split(':')[0]
            lifecycle_id = trade['trade_lifecycle_id']
            
            # Create user-friendly reason
            reason_map = {
                'entry_order_cancelled_no_position': 'Entry order was cancelled before filling',
                'no_entry_order_no_position': 'No entry order and no position opened',
                'old_pending_trade_no_position': 'Trade was pending too long without execution'
            }
            user_reason = reason_map.get(cancel_reason, cancel_reason)
            
            message = f"""❌ <b>Trade Cancelled</b>

📊 <b>Details:</b>
• Token: {token}
• Trade ID: {lifecycle_id[:8]}...
• Reason: {user_reason}

ℹ️ <b>Status:</b> Trade was automatically cancelled due to order issues
📱 Use /positions to view current positions"""
            
            await self.notification_manager.send_generic_notification(message.strip())
            logger.debug(f"📨 Sent cancellation notification for {symbol}")
            
        except Exception as e:
            logger.error(f"❌ Error sending cancellation notification for {symbol}: {e}")